/***************************************************************************
 *   Copyright (C) 2006 by Helio Chissini de Castro   *
 *   helio@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qlayout.h>
#include <qlabel.h>
#include <qdir.h>
#include <qfile.h>
#include <qheader.h>
#include <qlistview.h>
#include <qregexp.h>
#include <qpainter.h>

#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <klistview.h>
#include <kparts/genericfactory.h>
#include <kprocess.h>

#include "profilechooser.h"

typedef KGenericFactory<profilechooser, QWidget> profilechooserFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_profilechooser, profilechooserFactory("kcmprofilechooser"))
	
profilechooser::profilechooser(QWidget *parent, const char *name, const QStringList&)
    : KCModule(parent, name), myAboutData(0)
{
	QGridLayout *mainLayout = new QGridLayout( this, 1, 1, 11, 6, "MainLayout");
	
	defaultLabel = new QLabel( this );
	mainLayout->addWidget( defaultLabel, 0, 0 );
	
	availableProfiles = new KListView( this, "availableProfiles" );
	availableProfiles->addColumn("kde-config",-1);
	availableProfiles->header()->hide();
	availableProfiles->setRootIsDecorated(1);
	availableProfiles->setShowToolTips(1);
	availableProfiles->setItemsMovable(0);
	availableProfiles->setAcceptDrops(0);

	mainLayout->addWidget( availableProfiles, 1, 0 );
	
   QHBoxLayout *h = new QHBoxLayout( 0, 0, 2, "ButLayout" );
   KPushButton *bt = new KPushButton( i18n( "Default" ), this, "DefaultProfileButton" );
   connect( bt, SIGNAL( clicked() ), SLOT( setCurrent() ) );
   h->addWidget( bt );
   
   mainLayout->addLayout( h, 2, 0 );
   
   profileList.clear();
 
   setButtons( Apply|Default );
   
   setQuickHelp( i18n("<h1>Mandriva KDE Profile Chooser</h1>"
   	"Selects specific profiles to customize kde without touching main KDE configuration."
   	"Profiles are identified by a file calles \"mdvprofile\" in their root trees" ) ); 
   
   load();

   if (getuid() != 0){
	   availableProfiles->setEnabled(false);
	   bt->setEnabled(false);
   }
};


profilechooser::~profilechooser()
{
}

void profilechooser::defaults()
{
	ProfileItem * selected;
	QString profileName;
	
	// We need to know what is the product to
	// know the default profile
	QFileInfo file("/etc/product.id");
	if (file.isSymLink())
	{
		profileName = file.readLink();
		profileName = profileName.remove("/etc/");
		profileName.remove("product.id.");
		kdDebug() << "Default Profile Name: " << profileName << endl;
	}


	QListViewItemIterator it( availableProfiles);
	while ( it.current() )
	{   
		selected = dynamic_cast<ProfileItem*>(*it);
		if (!selected) return;

		if ( it.current()->text(0).contains(profileName) > 0 )
		{
			selected->setCurrent(true);
			availableProfiles->setSelected(it.current(), true);
		}else{  
			selected->setCurrent(false);
		}
		++it;
	}
	emit changed(true);
}


void profilechooser::load()
{
	QStringList lines;
	QString line, name, filemdv, SelectedProfile;
	QFile file, pName;
	ProfileItem *aux;
	bool current = false;
	QRegExp re("kderc$");

	QFileInfo profileFile("/etc/alternatives/kde-config");
	if ( profileFile.isSymLink() ){
		SelectedProfile = profileFile.readLink();
		kdDebug() << "Current Profile: " << SelectedProfile << endl;
	}
       
	file.setName("/var/lib/rpm/alternatives/kde-config");
	if ( !file.open(IO_ReadOnly) )
	{
		kdDebug() << "Error on open kde-config alternative" << endl;
		return;	
	}

	lines.clear();
	while ( !file.atEnd() )
	{
		if (!file.readLine(line, 9999) )
		{
			kdDebug() << "Error on reading lines of kde-config" << 
				  " alternative file" << endl;
			return;
		}
		line = line.remove('\n');
		if (line[0] != '/' || line.isEmpty() || line == "/etc/kderc" || line.endsWith("/upstream-kde-config") ) continue;
	        current = (line == SelectedProfile);
		filemdv = line;	
		pName.setName(filemdv.replace(re,"mdvprofile"));
		if (pName.open(IO_ReadOnly) )
		{
			pName.readLine(name,999);
			pName.close();
			name = name.remove('\n');
		
			lines.append(name);
			aux = new ProfileItem(availableProfiles,name);
			aux->setCurrent(current);
			profileList[name] = line;
			availableProfiles->insertItem(aux);
			kdDebug() << line << endl;
		}
	}
	file.close();

}

void profilechooser::save()
{
	QString profile;
	QString profileName;
	KProcess p; 

	kdDebug() << "OnSave" << endl;
	
	// Getting profile name
	profileName = availableProfiles->selectedItem()->text(0);
	// profile file
	profile = profileList[profileName];
	p << "update-alternatives" <<  "--set" <<  "kde-config";
	p << profile; 
        p.start(KProcess::Block);
	if (p.exitStatus() != 0)
		kdDebug() << "Error executing update-alternatives" << endl;	
}

void profilechooser::setCurrent()
{
    ProfileItem * selected;
    
    QListViewItemIterator it( availableProfiles);
    while ( it.current() )
    {
	selected = dynamic_cast<ProfileItem*>(*it);
	if (!selected) return;
		
	if ( it.current()->isSelected() )
	{
		selected->setCurrent(true); 
	   	 
	}else{
		selected->setCurrent(false);
	}
	++it;
    }

    emit changed(true);
}


ProfileItem::ProfileItem(QListView *parent, QString name)
: QListViewItem(parent,name)
{
	m_current = false;
		
}
void ProfileItem::paintCell ( QPainter * p, const QColorGroup & cg, int column, int width, int align )
{
	p->save();

	if (m_current)
	{
		QFont f = p->font();
		f.setBold(true);
		p->setFont(f);
	}
	QListViewItem::paintCell(p, cg, column, width, align);
	p->restore();	
}

#include "profilechooser.moc"
